﻿using Prism.Commands;
using Prism.Mvvm;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using Charting = System.Windows.Forms.DataVisualization.Charting;
using CsvHelper;
using CsvHelper.Configuration.Attributes;
using System.IO;
using System.Globalization;
using System.Windows;
using Word = Microsoft.Office.Interop.Word;
using System;

// https://www.c-sharpcorner.com/article/static-and-dynamic-line-chart-in-wpf-with-mvvm-pattern-using-prism-library/

namespace DontHarmDesktop.ViewModels.Reports
{
    class CostItem
    {
        [Name("Имя")]
        public string HumanName { get; set; }
        
        [Name("Priceness")]
        public int Priceness { get; set; }
    }

    public class EmployeeCostVM : BindableBase
    {
        /// <summary>
        /// Команда "Сформировать"
        /// </summary>
        public DelegateCommand StartCmd { get; set; }

        /// <summary>
        /// Команда "Экспортировать в CSV"
        /// </summary>
        public DelegateCommand ExportCsvCmd { get; set; }

        /// <summary>
        /// Команда "Экспортировать в DOCX"
        /// </summary>
        public DelegateCommand ExportDocxCmd { get; set; }

        /// <summary>
        /// Даные для графика
        /// </summary>
        private List<CostItem> Costs { get; set; }

        /// <summary>
        /// Интерфейс для отображения точек
        /// </summary>
        private readonly Charting.Series SeriesInterface;

        /// <summary>
        /// Фильтр по имени
        /// </summary>
        public string FilterName { get;  set; }

        /// <summary>
        /// Фильтр по ценности.
        /// 0: От 0 до 100
        /// 1: От 101 до 250
        /// 2: От 251
        /// </summary>
        public int PriceIndex { get; set; }

        public EmployeeCostVM(Charting.Series series) 
        {
            StartCmd = new DelegateCommand(StartExecuted);
            ExportCsvCmd = new DelegateCommand(ExportExecuted);
            ExportDocxCmd = new DelegateCommand(ExportDocxExecuted);
            Costs = new List<CostItem>();
            SeriesInterface = series;
            FilterName = "";
            PriceIndex = -1;
        }

        public void StartExecuted()
        {
            Refresh();
        }

        public void ExportExecuted()
        {
            // 1. Показать диалог выбора файла
            var ofd = new Microsoft.Win32.SaveFileDialog
            {
                Filter = "CSV файлы (*.csv)|*.csv|Все файлы (*.*)|*.*"
            };
            bool? selected = ofd.ShowDialog();
            if (selected == false)
            {
                return;
            }
            string fileName = ofd.FileName;

            // 2. Сгенерировать CSV
            using (var writer = new StreamWriter(fileName, false, Encoding.UTF8))
            {
                using (var csv = new CsvWriter(writer, CultureInfo.InvariantCulture))
                {
                    csv.WriteHeader<CostItem>();
                    csv.NextRecord();
                    Costs.ForEach(c =>
                    {
                        csv.WriteRecord(c);
                        csv.NextRecord();
                    });
                }
            }

            // 3. Profit
            MessageBox.Show("Экспорт успешен", "Завершение", MessageBoxButton.OK, MessageBoxImage.Information);
        }

        public void ExportDocxExecuted()
        {
            if (Costs.Count == 0)
            {
                return;
            }

            // 1. Показать диалог выбора файла
            var ofd = new Microsoft.Win32.SaveFileDialog
            {
                Filter = "DOCX файлы (*.docx)|*.docx|Все файлы (*.*)|*.*"
            };
            bool? selected = ofd.ShowDialog();
            if (selected == false)
            {
                return;
            }
            string fileName = ofd.FileName;

            // 2. Сгенерировать DOCX
            var app = new Word.Application();
            var doc = app.Documents.Add();

            // 2.1 Добавление заголовка
            var heading = doc.Paragraphs.Add();
            var headingRange = heading.Range;
            headingRange.Text = "Ценность сотрудников";
            heading.set_Style("Заголовок"); // зависит от текущего языка
            headingRange.InsertParagraphAfter();

            // 2.2 Добавление таблицы
            var tableP = doc.Paragraphs.Add();
            var tableRange = tableP.Range;
            var table = doc.Tables.Add(tableRange, Costs.Count + 1, 2);
            table.Borders.InsideLineStyle = Word.WdLineStyle.wdLineStyleSingle;
            table.Borders.OutsideLineStyle = Word.WdLineStyle.wdLineStyleSingle;
            table.Range.Cells.VerticalAlignment = Word.WdCellVerticalAlignment.wdCellAlignVerticalTop;

            // 2.3 Заголовок таблицы
            table.Cell(1, 1).Range.Text = "Сотрудник";
            table.Cell(1, 2).Range.Text = "Ценность в среднем";
            table.Rows[1].Range.Bold = 1;
            table.Rows[1].Range.ParagraphFormat.Alignment = Word.WdParagraphAlignment.wdAlignParagraphCenter;

            // 2.4 Данные таблицы
            int rowIndex = 2;
            CostItem maxPriceness = null;
            CostItem minPriceness = null;
            
            Costs.ForEach(c =>
            {
                if (rowIndex == 2)
                {
                    maxPriceness = c;
                    minPriceness = c;
                }
                else
                {
                    maxPriceness = c.Priceness > maxPriceness.Priceness ? c : maxPriceness;
                    minPriceness = c.Priceness < minPriceness.Priceness ? c : minPriceness;
                }
                
                table.Cell(rowIndex, 1).Range.Text = c.HumanName;
                table.Cell(rowIndex, 2).Range.Text = c.Priceness.ToString();
                
                rowIndex++;
            });

            // 2.5 Макс/мин
            var maxP = doc.Paragraphs.Add();
            var maxPRange = maxP.Range;
            maxPRange.Font.Color = Word.WdColor.wdColorBlueGray;
            maxPRange.Text = $"Сотрудник с наибольшей ценностью: {maxPriceness.HumanName} -- {maxPriceness.Priceness}";
            maxPRange.InsertParagraphAfter();
            
            var minP = doc.Paragraphs.Add();
            var minPRange = minP.Range;
            minPRange.Font.Color = Word.WdColor.wdColorBlueGray;
            minPRange.Text = $"Сотрудник с наименьшей ценностью: {minPriceness.HumanName} -- {minPriceness.Priceness}";

            // 3 Сохранение
            doc.SaveAs2(fileName);
            app.Visible = true;

            // 4. Profit
            MessageBox.Show("Экспорт успешен", "Завершение", MessageBoxButton.OK, MessageBoxImage.Information);
        }

        private async void Refresh()
        {
            var db = new Models.Entities();
            Costs.Clear();

            await db.users.ForEachAsync(u =>
            {
                // Фильтр имени пользователя
                if (!string.IsNullOrEmpty(FilterName) && !u.HumanName.ToLower().Contains(FilterName.ToLower()))
                {
                    return;
                }

                float priceness = u.Priceness;
                
                // Фильтр ценности
                int lowerPriceness;
                int higherPriceness;
                switch(PriceIndex)
                {
                    case 0:
                        lowerPriceness = 0;
                        higherPriceness = 100;
                        break;
                    case 1:
                        lowerPriceness = 101;
                        higherPriceness = 250;
                        break;
                    case 2:
                        lowerPriceness = 251;
                        higherPriceness = int.MaxValue;
                        break;
                    default:
                        lowerPriceness = 0;
                        higherPriceness = int.MaxValue;
                        break;
                }
                if (!(lowerPriceness < priceness && priceness < higherPriceness))
                {
                    return;
                }
                
                Costs.Add(new CostItem
                {
                    HumanName = u.HumanName,
                    Priceness = (int)priceness
                });
            });

            SeriesInterface.Points.Clear();
            Costs.ForEach(
                item => SeriesInterface.Points.AddXY(item.HumanName, item.Priceness)
            );
        }
    }
}
